#! /bin/bash

# Script to update Java connect strings as part of the database upgrade
# from Oracle 12c to 19c.  Script has a list of standard files to check
# and most likely update.  Script also has a list of directories to
# check for connect strings which aren't in the standard list of files.

# Files known to exist at test cells and which have connect strings.
# Not all of these files exist at every test cell.
FILES=(/cyflex/cmds/checkMSUconfig.sh \
/cyflex/cmds/find_database.sh \
/cyflex/cmds/java_application_setup \
/cyflex/system/checkMSUconfig.sh \
/cyflex/cell.def/cui_properties.txt \
/cyflex/specs.def/descriptions/extdatman.cmx.v2d \
/cyflex/specs.def/descriptions/extdatman.cummins.onga_load \
/cyflex/specs.def/descriptions/extdatman.cummins.weather \
/cyflex/specs.def/descriptions/prod_exec_properties.txt \
/cyflex/specs.def/docs/trs_installation_configuration.html \
/cyflex/specs.def/samples/extdatman.cmx.v2d \
/cyflex/specs.def/samples/extdatman.cummins.onga \
/cyflex/specs.def/samples/extdatman.cummins.onga_load \
/cyflex/specs.def/samples/extdatman.cummins.weather \
/cyflex/specs.def/samples/prod_exec_properties.txt \
/cyflex/specs.def/samples/system_log_replicate.properties \
/cell/cui_properties.txt \
/specs/extdatman.cummins.weather \
/specs/extdatman.cummins.onga \
/opt/sys_log_replicate/properties.txt)

# Class path where the .class or .jar file for the Java update
# application is located.
export CLASSPATH=/usr/local/share/cyflex/jar/

# Directories where files with connect strings might exist which aren't
# in the above list.
DIRECTORIES=(/specs /cell /cyflex)

# Suffix to append to the backup version of any file being updated.
SUFFIX=jdbc_update

# Function to update a file containing a connect string
function update_file {
   java jdbc_update $1 $2 UPDATE
}

# See if a file needs an update.  It may have already been updated, for
# example if this script is run twice at a test cell.
function check_update_needed {
   java jdbc_update $1 $1 CHECK
   return $?
}

# Function to get the current owner, group and permissions for a file.
function get_file_info {
   PERMISSION=$(stat -c "%a" $1)
   OWNER=$(stat -c "%U" $1)
   GROUP=$(stat -c "%G" $1)
}

# Function to restore owner, group and permissions to a file
function restore_file_info {
   chown ${OWNER}:${GROUP} $1
   chmod $PERMISSION $1
}

echo
echo Processing known files
for FILE in ${FILES[@]}; do
   if [ -e $FILE ]; then
      check_update_needed $FILE
      if (( $? == 1 )); then
         # Get file information to apply to files after updating
         get_file_info $FILE
         # Create backup copy
         cp $FILE ${FILE}.$SUFFIX
         # Use the backup as the input and the original as the new/out
         # file
         update_file ${FILE}.$SUFFIX $FILE
         # Put owner, group and permissions back on both files
         restore_file_info $FILE
         restore_file_info ${FILE}.$SUFFIX
      fi
   fi
done

# Scan standard directories to see if there are any files with connect
# strings which aren't in the standard file list.
echo
echo Processing directories for unknown files
# Loop through all listed directories
for DIRECTORY in ${DIRECTORIES[@]}
do
   if [ -e $DIRECTORY ]; then
      echo
      echo Searching $DIRECTORY
      # Find files with connect strings.
      grep -lr jdbc:oracle:thin $DIRECTORY > /tmp/jdbc_update.lst
      # Loop through all files with connect strings in directory.
      while read FOUND_FILE
      do
         # For each file with a connect string, see if the file is in
         # the standard list.
         FOUND=0
         for FILE in ${FILES[@]}; do
            # File is in the standard list and doesn't need to be
            # processed.
            if [ $FOUND_FILE == $FILE ]; then
               FOUND=1
            fi
         done
         # Not in standard list - notify the user.
         if [ $FOUND == 0 ]; then
            # We don't need to report backed up versions of changed
            # files
            NEW_FILE=${FOUND_FILE%.jdbc_update}
            if [ "$NEW_FILE" = "$FOUND_FILE" ] ; then
               echo $FOUND_FILE
            fi
         fi
      done </tmp/jdbc_update.lst
   fi
done

rm -f /tmp/jdbc_update.lst
